unit Interpreter;

interface
uses
  SysUtils,Classes,Dialogs,Forms, Graphics;

type
  TInterpreter = class(TThread)
  private
    cmdtokens:TStringList;
    info, Line:String;
    FPutText:String;
    FPutTextColor:TColor;
    Code:integer;
    FOnUsersChanged:TNotifyEvent;
    FPong:TNotifyEvent;
    procedure MakeTokens;
    procedure PutText;
    procedure Interprete;
    Function FromToken(nr:integer; strip:boolean):String;
  protected
    procedure Execute; override;
  public
    CurrentChannel:String;
    OwnNick:String;
    property PONG:TNotifyEvent read FPong write FPong;
    property OnUsersChanged:TNotifyEvent read FOnUsersChanged write FOnUsersChanged;
    Constructor Create;
    Destructor Destroy;override;
  end;

implementation
uses Windows, Glob, Messages, main;

procedure TInterpreter.PutText;
var oldcount:integer;
Begin
  with MainReceived do
  begin
    oldcount:=Lines.Count;
    SelStart:=length(Text);
    SelAttributes.Color:=FPutTextColor;
    SelText:=FPutText+#13#10;

    if Lines.count>100 then
    begin
      Lines.Delete(0);
    end;
    oldcount:=Lines.Count-oldcount;
    repeat
      PostMessage(MainReceived.Handle,EM_SCROLL,SB_LINEDOWN,0);
      dec(oldcount);
    until oldcount<=0;
  end;
End;

procedure TInterpreter.Interprete;
var i:integer;
Begin
  if Code=-1 then exit;
  FPutTextColor:=clNormal;

  case Code of
  0:
    begin
      FPutText:=FromToken(0, false);
      Synchronize(PutText);
    end;
  2010:
    begin
      OwnNick:=FromToken(2, false);
      FPutTextColor:=clInfo;
      FPutText:='** Your nick is now '+OwnNick;
      Synchronize(PutText);
    end;
  1007:
    begin
      CurrentChannel:=FromToken(2, true);
      FPutText:='** Now talking in channel '+CurrentChannel;
      FPutTextColor:=clJoinedChannel;
      Synchronize(PutText);
      Users.Clear;
      if Assigned(FOnUsersChanged) then FOnUsersChanged(Self);
    end;
  1001, 1002:
    begin
      Users.Add(cmdTokens.Strings[2]);
      if code=1002 then
      begin
        FPutTextColor:=clUserJoinChannel;
        FPutText:='** '+Users.Strings[Users.Count-1]+' joined';
        Synchronize(PutText);
      end;
      if Assigned(FOnUsersChanged) then FOnUsersChanged(Self);
    end;
  1009:
    begin
      FPutTextColor:=clUserJoinChannel;
      FPutText:='** '+cmdTokens.Strings[2]+' has got the hammer...';
      Synchronize(PutText);
    end;
  1003:
    begin
      i:=Users.indexof(cmdTokens.Strings[2]);
      if i>=0 then
      begin
        FPutTextColor:=clUserPartChannel;
        FPutText:='** '+Users.Strings[i]+' parted';
        Synchronize(PutText);
        Users.Delete(i);
      end;
      if Assigned(FOnUsersChanged) then FOnUsersChanged(Self);
    end;
  1018:
    begin
      FPutTextColor:=clInfo;
      FPutText:='** INFO: '+FromToken(2, true);
      Synchronize(PutText);
    end;
  1006:
    begin
      FPutTextColor:=clInfo;
      FPutText:='** BROADCAST: '+FromToken(2, true);
      Synchronize(PutText);
    end;
  1019:
    begin
      FPutTextColor:=clError;
      FPutText:='** ERROR: '+FromToken(2, true);
      Synchronize(PutText);
    end;
  1005:
    begin
      FPutTextColor:=clTalk;
      FPutText:='<'+cmdTokens.Strings[2]+'> '+FromToken(4, true);
      Synchronize(PutText);
    end;
  1010,1004:
    begin
      FPutTextColor:=clWhisper;
      if code=1010 then FPutText:='<To: ' else FPutText:='<From: ';
      FPutText:=FPutText+cmdTokens.Strings[2]+'> '+FromToken(4, true);
      Synchronize(PutText);
      if PopupPrivate then
      begin
        if IsIconic(MainForm.Handle) then Application.Restore;
        SetWindowPos(MainForm.Handle, HWND_TOPMOST, 0,0,0,0,SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
        Application.ProcessMessages;
        SetWindowPos(MainForm.Handle, HWND_TOP, 0,0,0,0,SWP_NOMOVE+SWP_NOSIZE+SWP_SHOWWINDOW);
        SetForegroundWindow(MainForm.Handle);
      end;
    end;
  1023:
    begin
      FPutTextColor:=clEmote;
      FPutText:='<'+cmdTokens.Strings[2]+' '+FromToken(4, true)+'>';
      Synchronize(PutText);
    end;
  1015:
    begin
      FPutTextColor:=clError;
      FPutText:='** Channel is restricted '+FromToken(2, false);
      Synchronize(PutText);
    end;
  1013:
    begin
      FPutTextColor:=clError;
      FPutText:='** Channel is full '+FromToken(2, false);
      Synchronize(PutText);
    end;
  1014:
    begin
      FPutTextColor:=clError;
      FPutText:='** Channel doesn''t exist '+FromToken(2, false);
      Synchronize(PutText);
    end;
  2000:
    begin
      FPong(Self);
    end;
  else
    begin
      FPutText:=FromToken(0, false);
      Synchronize(PutText);
    end;
  end;
End;

procedure TInterpreter.Execute;
Begin
  Repeat
    if Received.Count>0 then
    begin
      Line:=Received.Strings[0];
      Received.Delete(0);
      MakeTokens;
      Interprete;
    end
    else
      Sleep(100);
  until Terminated;
End;

procedure TInterpreter.MakeTokens;
var i:integer;
Begin
  cmdtokens.clear;
  repeat
    i:=pos(' ',Line);
    if (i>0) then
    begin
      info:=copy(Line,1,i-1);
      if info<>'' then cmdtokens.Add(info);
      Delete(Line,1,i);
    end
    else
    begin
      if Line<>'' then
        cmdtokens.Add(Line);
      Line:='';
    end;
  until Line='';
  if (cmdTokens.Count>0) then
  begin
    val(cmdTokens.Strings[0], Code, i);
    if i>0 then Code:=-1;
  end
  else
    Code:=-1;
End;
Function TInterpreter.FromToken(nr:integer; strip:boolean):String;
Begin
  Result:='';
  while (nr < cmdTokens.Count) do
  begin
    Result:=Result+cmdTokens.Strings[nr];
    inc(nr);
    if nr < cmdTokens.Count then Result:=Result+' ';
  end;
  if Strip then
  begin
    if (Result[1]='"') then delete(Result, 1, 1);
    if (Result[Length(Result)]='"') then delete(Result, Length(Result), 1);
  end;
End;

Constructor TInterpreter.Create;
Begin
  cmdtokens:=TStringList.Create;
  Received:=TStringList.Create;
  Users:=TStringList.Create;
  OwnNick:='';
  inherited Create(False);
End;

Destructor TInterpreter.Destroy;
Begin
  cmdtokens.Free;
  Users.Free;
  Received.free;
  inherited Destroy;
End;

end.
 